@@ -1,42 +1,31 @@
 
 /*---------------------------------------------------------------*/
-/*---                                                         ---*/
-/*--- This file (libvex_guest_ppc64.h) is                     ---*/
-/*--- Copyright (C) OpenWorks LLP.  All rights reserved.      ---*/
-/*---                                                         ---*/
+/*--- begin                              libvex_guest_ppc64.h ---*/
 /*---------------------------------------------------------------*/
 
 /*
-   This file is part of LibVEX, a library for dynamic binary
-   instrumentation and translation.
+   This file is part of Valgrind, a dynamic binary instrumentation
+   framework.
 
-   Copyright (C) 2004-2007 OpenWorks LLP.  All rights reserved.
+   Copyright (C) 2004-2010 OpenWorks LLP
+      info@open-works.net
 
-   This library is made available under a dual licensing scheme.
+   This program is free software; you can redistribute it and/or
+   modify it under the terms of the GNU General Public License as
+   published by the Free Software Foundation; either version 2 of the
+   License, or (at your option) any later version.
 
-   If you link LibVEX against other code all of which is itself
-   licensed under the GNU General Public License, version 2 dated June
-   1991 ("GPL v2"), then you may use LibVEX under the terms of the GPL
-   v2, as appearing in the file LICENSE.GPL.  If the file LICENSE.GPL
-   is missing, you can obtain a copy of the GPL v2 from the Free
-   Software Foundation Inc., 51 Franklin St, Fifth Floor, Boston, MA
+   This program is distributed in the hope that it will be useful, but
+   WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+   General Public License for more details.
+
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
    02110-1301, USA.
 
-   For any other uses of LibVEX, you must first obtain a commercial
-   license from OpenWorks LLP.  Please contact info@open-works.co.uk
-   for information about commercial licensing.
-
-   This software is provided by OpenWorks LLP "as is" and any express
-   or implied warranties, including, but not limited to, the implied
-   warranties of merchantability and fitness for a particular purpose
-   are disclaimed.  In no event shall OpenWorks LLP be liable for any
-   direct, indirect, incidental, special, exemplary, or consequential
-   damages (including, but not limited to, procurement of substitute
-   goods or services; loss of use, data, or profits; or business
-   interruption) however caused and on any theory of liability,
-   whether in contract, strict liability, or tort (including
-   negligence or otherwise) arising in any way out of the use of this
-   software, even if advised of the possibility of such damage.
+   The GNU General Public License is contained in the file COPYING.
 
    Neither the names of the U.S. Department of Energy nor the
    University of California nor the names of its contributors may be
@@ -166,38 +155,41 @@ typedef
       /* 504 */ ULong guest_FPR31;
 
       // Vector Registers
-      /*  512 */ U128 guest_VR0 __attribute__ ((aligned (16)));
-      /*  528 */ U128 guest_VR1 __attribute__ ((aligned (16)));
-      /*  544 */ U128 guest_VR2 __attribute__ ((aligned (16)));
-      /*  560 */ U128 guest_VR3 __attribute__ ((aligned (16)));
-      /*  576 */ U128 guest_VR4 __attribute__ ((aligned (16)));
-      /*  592 */ U128 guest_VR5 __attribute__ ((aligned (16)));
-      /*  608 */ U128 guest_VR6 __attribute__ ((aligned (16)));
-      /*  624 */ U128 guest_VR7 __attribute__ ((aligned (16)));
-      /*  640 */ U128 guest_VR8 __attribute__ ((aligned (16)));
-      /*  656 */ U128 guest_VR9 __attribute__ ((aligned (16)));
-      /*  672 */ U128 guest_VR10 __attribute__ ((aligned (16)));
-      /*  688 */ U128 guest_VR11 __attribute__ ((aligned (16)));
-      /*  704 */ U128 guest_VR12 __attribute__ ((aligned (16)));
-      /*  720 */ U128 guest_VR13 __attribute__ ((aligned (16)));
-      /*  736 */ U128 guest_VR14 __attribute__ ((aligned (16)));
-      /*  752 */ U128 guest_VR15 __attribute__ ((aligned (16)));
-      /*  768 */ U128 guest_VR16 __attribute__ ((aligned (16)));
-      /*  784 */ U128 guest_VR17 __attribute__ ((aligned (16)));
-      /*  800 */ U128 guest_VR18 __attribute__ ((aligned (16)));
-      /*  816 */ U128 guest_VR19 __attribute__ ((aligned (16)));
-      /*  832 */ U128 guest_VR20 __attribute__ ((aligned (16)));
-      /*  848 */ U128 guest_VR21 __attribute__ ((aligned (16)));
-      /*  864 */ U128 guest_VR22 __attribute__ ((aligned (16)));
-      /*  880 */ U128 guest_VR23 __attribute__ ((aligned (16)));
-      /*  896 */ U128 guest_VR24 __attribute__ ((aligned (16)));
-      /*  912 */ U128 guest_VR25 __attribute__ ((aligned (16)));
-      /*  928 */ U128 guest_VR26 __attribute__ ((aligned (16)));
-      /*  944 */ U128 guest_VR27 __attribute__ ((aligned (16)));
-      /*  960 */ U128 guest_VR28 __attribute__ ((aligned (16)));
-      /*  976 */ U128 guest_VR29 __attribute__ ((aligned (16)));
-      /*  992 */ U128 guest_VR30 __attribute__ ((aligned (16)));
-      /* 1008 */ U128 guest_VR31 __attribute__ ((aligned (16)));
+      // IMPORTANT: the user of libvex must place the guest state so as
+      // to ensure that guest_VR{0..31}, and any shadows thereof, are
+      // 16-aligned.
+      /*  512 */ U128 guest_VR0;
+      /*  528 */ U128 guest_VR1;
+      /*  544 */ U128 guest_VR2;
+      /*  560 */ U128 guest_VR3;
+      /*  576 */ U128 guest_VR4;
+      /*  592 */ U128 guest_VR5;
+      /*  608 */ U128 guest_VR6;
+      /*  624 */ U128 guest_VR7;
+      /*  640 */ U128 guest_VR8;
+      /*  656 */ U128 guest_VR9;
+      /*  672 */ U128 guest_VR10;
+      /*  688 */ U128 guest_VR11;
+      /*  704 */ U128 guest_VR12;
+      /*  720 */ U128 guest_VR13;
+      /*  736 */ U128 guest_VR14;
+      /*  752 */ U128 guest_VR15;
+      /*  768 */ U128 guest_VR16;
+      /*  784 */ U128 guest_VR17;
+      /*  800 */ U128 guest_VR18;
+      /*  816 */ U128 guest_VR19;
+      /*  832 */ U128 guest_VR20;
+      /*  848 */ U128 guest_VR21;
+      /*  864 */ U128 guest_VR22;
+      /*  880 */ U128 guest_VR23;
+      /*  896 */ U128 guest_VR24;
+      /*  912 */ U128 guest_VR25;
+      /*  928 */ U128 guest_VR26;
+      /*  944 */ U128 guest_VR27;
+      /*  960 */ U128 guest_VR28;
+      /*  976 */ U128 guest_VR29;
+      /*  992 */ U128 guest_VR30;
+      /* 1008 */ U128 guest_VR31;
 
       /* 1024 */ ULong guest_CIA;    // IP (no arch visible register)
       /* 1032 */ ULong guest_LR;     // Link Register
@@ -246,28 +238,32 @@ typedef
       /* 1088 */ ULong guest_TISTART;
       /* 1096 */ ULong guest_TILEN;
 
-      /* For lwarx/stwcx.: 0 == no reservation exists, non-0 == a
-         reservation exists. */
-      /* 1104 */ ULong guest_RESVN;
-
       /* Used to record the unredirected guest address at the start of
          a translation whose start has been redirected.  By reading
          this pseudo-register shortly afterwards, the translation can
          find out what the corresponding no-redirection address was.
          Note, this is only set for wrap-style redirects, not for
          replace-style ones. */
-      /* 1112 */ ULong guest_NRADDR;
-      /* 1120 */ ULong guest_NRADDR_GPR2;
+      /* 1104 */ ULong guest_NRADDR;
+      /* 1112 */ ULong guest_NRADDR_GPR2;
 
      /* A grows-upwards stack for hidden saves/restores of LR and R2
         needed for function interception and wrapping on ppc64-linux.
         A horrible hack.  REDIR_SP points to the highest live entry,
         and so starts at -1. */
-      /* 1128 */ ULong guest_REDIR_SP;
-      /* 1136 */ ULong guest_REDIR_STACK[VEX_GUEST_PPC64_REDIR_STACK_SIZE];
+      /* 1120 */ ULong guest_REDIR_SP;
+      /* 1128 */ ULong guest_REDIR_STACK[VEX_GUEST_PPC64_REDIR_STACK_SIZE];
+
+      /* Needed for AIX: CIA at the last SC insn.  Used when backing up
+         to restart a syscall that has been interrupted by a signal. */
+      /* 1384 */ ULong guest_IP_AT_SYSCALL; 
+
+      /* SPRG3, which AIUI is readonly in user space.  Needed for
+         threading on AIX. */
+      /* ???? */ ULong guest_SPRG3_RO;
 
       /* Padding to make it have an 8-aligned size */
-      /* UInt  padding; */
+      /* ???? */ ULong padding2;
    }
    VexGuestPPC64State;
 
