@@ -7,7 +7,7 @@
    This file is part of Valgrind, a dynamic binary instrumentation
    framework.
 
-   Copyright (C) 2000-2007 Julian Seward
+   Copyright (C) 2000-2010 Julian Seward
       jseward@acm.org
 
    This program is free software; you can redistribute it and/or
@@ -39,60 +39,111 @@
 
 #include "pub_tool_debuginfo.h"
 
+/* Initialise the entire module.  Must be called first of all. */
+extern void VG_(di_initialise) ( void );
+
 /* LINUX: Notify the debuginfo system about a new mapping, or the
    disappearance of such, or a permissions change on an existing
    mapping.  This is the way new debug information gets loaded.  If
    allow_SkFileV is True, it will try load debug info if the mapping
    at 'a' belongs to Valgrind; whereas normally (False) it will not do
    that.  This allows us to carefully control when the thing will read
-   symbols from the Valgrind executable itself. */
-#if defined(VGO_linux)
-extern void VG_(di_notify_mmap)( Addr a, Bool allow_SkFileV );
+   symbols from the Valgrind executable itself.
+
+   If a call to VG_(di_notify_mmap) causes debug info to be read, then
+   the returned ULong is an abstract handle which can later be used to
+   refer to the debuginfo read as a result of this specific mapping,
+   in later queries to m_debuginfo.  In this case the handle value
+   will be one or above.  If the returned value is zero, no debug info
+   was read. */
+#if defined(VGO_linux)  ||  defined(VGO_darwin)
+extern ULong VG_(di_notify_mmap)( Addr a, Bool allow_SkFileV );
 
 extern void VG_(di_notify_munmap)( Addr a, SizeT len );
 
 extern void VG_(di_notify_mprotect)( Addr a, SizeT len, UInt prot );
+
+/* this should really return ULong, as per VG_(di_notify_mmap). */
+extern void VG_(di_notify_pdb_debuginfo)( Int fd, Addr avma,
+                                          SizeT total_size,
+                                          PtrdiffT unknown_purpose__reloc );
 #endif
 
 #if defined(VGO_aix5)
+// GrP fixme use this instead for darwin?
 /* AIX5: Very similar, except packaged more neatly.  The supplied
    parameters describe a code segment and its associated data segment,
    that have recently been mapped in -- so we need to read debug info
    for it -- or conversely, have recently been dumped, in which case
-   the relevant debug info has to be unloaded. */
-extern void VG_(di_aix5_notify_segchange)( 
-               Addr   code_start,
-               Word   code_len,
-               Addr   data_start,
-               Word   data_len,
-               UChar* file_name,
-               UChar* mem_name,
-               Bool   is_mainexe,
-               Bool   acquire
-            );
+   the relevant debug info has to be unloaded.
+
+   The returned ULong has the same meaning as documented for
+   VG_(di_notify_mmap) just above. */
+extern ULong VG_(di_aix5_notify_segchange)( 
+                Addr   code_start,
+                Word   code_len,
+                Addr   data_start,
+                Word   data_len,
+                UChar* file_name,
+                UChar* mem_name,
+                Bool   is_mainexe,
+                Bool   acquire
+             );
 #endif
 
-extern Bool VG_(get_fnname_nodemangle)( Addr a, 
-                                        Char* fnname, Int n_fnname );
+extern void VG_(di_discard_ALL_debuginfo)( void );
 
-/* Use DWARF2/3 CFA information to do one step of stack unwinding. */
-extern Bool VG_(use_CF_info) ( /*MOD*/Addr* ipP,
-                               /*MOD*/Addr* spP,
-                               /*MOD*/Addr* fpP,
+/* Like VG_(get_fnname), but it does not do C++ demangling nor Z-demangling
+ * nor below-main renaming.
+ * It should not be used for any names that will be shown to users.
+ * It should only be used in cases where the names of interest will have
+ * particular (ie. non-mangled) forms, or the mangled form is acceptable. */
+extern
+Bool VG_(get_fnname_raw) ( Addr a, Char* buf, Int nbuf );
+
+/* Like VG_(get_fnname), but without C++ demangling.  (But it does
+ * Z-demangling and below-main renaming.) */
+extern
+Bool VG_(get_fnname_no_cxx_demangle) ( Addr a, Char* buf, Int nbuf );
+
+
+/* Use DWARF2/3 CFA information to do one step of stack unwinding.
+   D3UnwindRegs holds the current register values, and is
+   arch-specific.  Note that the x86 and amd64 definitions are shared
+   and so the regs are named 'xip' etc rather than 'eip' and 'rip'. */
+#if defined(VGA_amd64) || defined(VGA_x86)
+typedef
+   struct { Addr xip; Addr xsp; Addr xbp; }
+   D3UnwindRegs;
+#elif defined(VGA_arm)
+typedef
+   struct { Addr r15; Addr r14; Addr r13; Addr r12; Addr r11; Addr r7; }
+   D3UnwindRegs;
+#elif defined(VGA_ppc32) || defined(VGA_ppc64)
+typedef
+   UChar  /* should be void, but gcc complains at use points */
+   D3UnwindRegs;
+#else
+#  error "Unsupported arch"
+#endif
+
+extern Bool VG_(use_CF_info) ( /*MOD*/D3UnwindRegs* uregs,
                                Addr min_accessible,
                                Addr max_accessible );
 
+
+/* Use MSVC FPO data to do one step of stack unwinding. */
+extern Bool VG_(use_FPO_info) ( /*MOD*/Addr* ipP,
+                                /*MOD*/Addr* spP,
+                                /*MOD*/Addr* fpP,
+                                Addr min_accessible,
+                                Addr max_accessible );
+
 /* ppc64-linux only: find the TOC pointer (R2 value) that should be in
    force at the entry point address of the function containing
    guest_code_addr.  Returns 0 if not known. */
 extern Addr VG_(get_tocptr) ( Addr guest_code_addr );
 
-/* This is only available to core... don't demangle C++ names, but do
-   do Z-demangling, match anywhere in function, and don't show
-   offsets. */
-extern
-Bool VG_(get_fnname_Z_demangle_only) ( Addr a, Char* buf, Int nbuf );
-
 /* Map a function name to its entry point and toc pointer.  Is done by
    sequential search of all symbol tables, so is very slow.  To
    mitigate the worst performance effects, you may specify a soname
