@@ -7,7 +7,7 @@
    This file is part of Valgrind, a dynamic binary instrumentation
    framework.
 
-   Copyright (C) 2000-2009 Julian Seward
+   Copyright (C) 2000-2010 Julian Seward
       jseward@acm.org
 
    This program is free software; you can redistribute it and/or
@@ -53,32 +53,32 @@
    readable, at least.  Otherwise Memcheck complains we're jumping to
    invalid addresses. */
 
-extern void VG_(trampoline_stuff_start);
-extern void VG_(trampoline_stuff_end);
+extern Addr VG_(trampoline_stuff_start);
+extern Addr VG_(trampoline_stuff_end);
 
 #if defined(VGP_x86_linux)
-extern void VG_(x86_linux_SUBST_FOR_sigreturn);
-extern void VG_(x86_linux_SUBST_FOR_rt_sigreturn);
+extern Addr VG_(x86_linux_SUBST_FOR_sigreturn);
+extern Addr VG_(x86_linux_SUBST_FOR_rt_sigreturn);
 extern Char* VG_(x86_linux_REDIR_FOR_index) ( const Char*, Int );
 #endif
 
 #if defined(VGP_amd64_linux)
-extern void VG_(amd64_linux_SUBST_FOR_rt_sigreturn);
-extern void VG_(amd64_linux_REDIR_FOR_vgettimeofday);
-extern void VG_(amd64_linux_REDIR_FOR_vtime);
+extern Addr VG_(amd64_linux_SUBST_FOR_rt_sigreturn);
+extern Addr VG_(amd64_linux_REDIR_FOR_vgettimeofday);
+extern Addr VG_(amd64_linux_REDIR_FOR_vtime);
 extern UInt VG_(amd64_linux_REDIR_FOR_strlen)( void* );
 #endif
 
 #if defined(VGP_ppc32_linux)
-extern void  VG_(ppc32_linux_SUBST_FOR_sigreturn);
-extern void  VG_(ppc32_linux_SUBST_FOR_rt_sigreturn);
+extern Addr  VG_(ppc32_linux_SUBST_FOR_sigreturn);
+extern Addr  VG_(ppc32_linux_SUBST_FOR_rt_sigreturn);
 extern UInt  VG_(ppc32_linux_REDIR_FOR_strlen)( void* );
 extern UInt  VG_(ppc32_linux_REDIR_FOR_strcmp)( void*, void* );
 extern void* VG_(ppc32_linux_REDIR_FOR_strchr)( void*, Int );
 #endif
 
 #if defined(VGP_ppc64_linux)
-extern void  VG_(ppc64_linux_SUBST_FOR_rt_sigreturn);
+extern Addr  VG_(ppc64_linux_SUBST_FOR_rt_sigreturn);
 extern UInt  VG_(ppc64_linux_REDIR_FOR_strlen)( void* );
 extern void* VG_(ppc64_linux_REDIR_FOR_strchr)( void*, Int );
 /* A label (sans dot) marking the ultra-magical return stub via which
@@ -88,7 +88,13 @@ extern void* VG_(ppc64_linux_REDIR_FOR_strchr)( void*, Int );
    restore the thread's LR and R2 registers from a small stack in the
    ppc64 guest state structure, and then branch to LR.  Convoluted?
    Confusing?  You betcha.  Could I think of anything simpler?  No. */
-extern void VG_(ppctoc_magic_redirect_return_stub);
+extern Addr VG_(ppctoc_magic_redirect_return_stub);
+#endif
+
+#if defined(VGP_arm_linux)
+extern UInt  VG_(arm_linux_REDIR_FOR_strlen)( void* );
+//extern void* VG_(arm_linux_REDIR_FOR_index) ( void*, Int );
+extern void* VG_(arm_linux_REDIR_FOR_memcpy)( void*, void*, Int );
 #endif
 
 #if defined(VGP_ppc32_aix5)
@@ -99,28 +105,39 @@ extern void VG_(ppctoc_magic_redirect_return_stub);
    then it cleans up the register state to be more what it really
    should be at client startup, and finally it jumps to the client's
    real entry point. */
-extern void VG_(ppc32_aix5_do_preloads_then_start_client);
+extern Addr VG_(ppc32_aix5_do_preloads_then_start_client);
 
 /* See comment for VG_(ppctoc_magic_redirect_return_stub) above. */
-extern void VG_(ppctoc_magic_redirect_return_stub);
+extern Addr VG_(ppctoc_magic_redirect_return_stub);
 #endif
 
 #if defined(VGP_ppc64_aix5)
 /* See comment for VG_(ppctoc_magic_redirect_return_stub) above. */
-extern void VG_(ppctoc_magic_redirect_return_stub);
+extern Addr VG_(ppctoc_magic_redirect_return_stub);
 
 /* See comment for ppc32_aix5 equivalent above. */
-extern void VG_(ppc64_aix5_do_preloads_then_start_client);
+extern Addr VG_(ppc64_aix5_do_preloads_then_start_client);
 #endif
 
-#if defined(VGO_darwin)
-extern void  VG_(x86_darwin_SUBST_FOR_sigreturn);
-extern SizeT VG_(darwin_REDIR_FOR_strlen)( void* );
-extern SizeT VG_(darwin_REDIR_FOR_strcmp)( void*, void* );
-extern void* VG_(darwin_REDIR_FOR_strcat)( void*, void * );
-extern char* VG_(darwin_REDIR_FOR_strcpy)( char *s1, char *s2 );
-extern SizeT VG_(darwin_REDIR_FOR_strlcat)( char *s1, const char *s2, SizeT size );
-extern UInt VG_(darwin_REDIR_FOR_arc4random)( void );
+#if defined(VGP_x86_darwin)
+extern Addr  VG_(x86_darwin_SUBST_FOR_sigreturn);
+extern SizeT VG_(x86_darwin_REDIR_FOR_strlen)( void* );
+extern SizeT VG_(x86_darwin_REDIR_FOR_strcmp)( void*, void* );
+extern void* VG_(x86_darwin_REDIR_FOR_strcat)( void*, void * );
+extern char* VG_(x86_darwin_REDIR_FOR_strcpy)( char *s1, char *s2 );
+extern SizeT VG_(x86_darwin_REDIR_FOR_strlcat)( char *s1, const char *s2,
+                                                SizeT size );
+#endif
+
+#if defined(VGP_amd64_darwin)
+extern Addr  VG_(amd64_darwin_SUBST_FOR_sigreturn);
+extern SizeT VG_(amd64_darwin_REDIR_FOR_strlen)( void* );
+extern SizeT VG_(amd64_darwin_REDIR_FOR_strcmp)( void*, void* );
+extern void* VG_(amd64_darwin_REDIR_FOR_strcat)( void*, void * );
+extern char* VG_(amd64_darwin_REDIR_FOR_strcpy)( char *s1, char *s2 );
+extern SizeT VG_(amd64_darwin_REDIR_FOR_strlcat)( char *s1, const char *s2,
+                                                  SizeT size );
+extern UInt VG_(amd64_darwin_REDIR_FOR_arc4random)( void );
 #endif
 
 #endif   // __PUB_CORE_TRAMPOLINE_H
