@@ -7,7 +7,7 @@
    This file is part of Valgrind, a dynamic binary instrumentation
    framework.
 
-   Copyright (C) 2000-2006 Julian Seward 
+   Copyright (C) 2000-2010 Julian Seward 
       jseward@acm.org
 
    This program is free software; you can redistribute it and/or
@@ -33,43 +33,11 @@
 
 //--------------------------------------------------------------------
 // PURPOSE: This module implements user-mode execve, ie. program loading
-// and exec'ing.  It is shared between stage1 and stage2.
+// and exec'ing.
 //--------------------------------------------------------------------
 
-#include <elf.h>
-#include <sys/types.h>
-
 /*------------------------------------------------------------*/
-/*--- General stuff                                        ---*/
-/*------------------------------------------------------------*/
-
-/* This is only here so it can be shared between stage1 and stage2 */
-
-/* JRS 9 Aug 05: both of these are apparently unused, except by
-   memcheck/tests/vgtest_ume.c. */
-//zz extern
-//zz void VG_(foreach_map)(int (*fn)(char *start, char *end,
-//zz 			        const char *perm, off_t offset,
-//zz 			        int maj, int min, int ino, void* extra),
-//zz                       void* extra);
-//zz 
-//zz /* Jump to 'dst', but first set the stack pointer to 'stack'.  Also,
-//zz    clear all the integer registers before entering 'dst'.  It's
-//zz    important that the stack pointer is set to exactly 'stack' and not
-//zz    (eg) stack - apparently_harmless_looking_small_offset.  Basically
-//zz    because the code at 'dst' might be wanting to scan the area above
-//zz    'stack' (viz, the auxv array), and putting spurious words on the
-//zz    stack confuses it.
-//zz 
-//zz    This is only exported so that vgtest_ume.c can use it.
-//zz */
-//zz extern
-//zz __attribute__((noreturn))
-//zz void VG_(jump_and_switch_stacks) ( Addr stack, Addr dst );
-
-
-/*------------------------------------------------------------*/
-/*--- Loading ELF files                                    ---*/
+/*--- Loading files                                        ---*/
 /*------------------------------------------------------------*/
 
 // Info needed to load and run a program.  IN/INOUT/OUT refers to the
@@ -81,9 +49,18 @@ typedef
       Addr exe_base;     // INOUT: lowest (allowed) address of exe
       Addr exe_end;      // INOUT: highest (allowed) address
 
+#if !defined(VGO_darwin)
       Addr phdr;         // OUT: address phdr was mapped at
       Int  phnum;        // OUT: number of phdrs
       Addr interp_base;  // OUT: where interpreter (ld.so) was mapped
+#else
+      Addr  stack_start;      // OUT: address of start of stack segment (hot)
+      Addr  stack_end;        // OUT: address of end of stack segment (cold)
+      Addr  text;             // OUT: address of executable's Mach header
+      Bool  dynamic;          // OUT: False iff executable is static
+      char* executable_path;  // OUT: path passed to execve()
+#endif
+
       Addr entry;        // OUT: entrypoint in main executable
       Addr init_ip;      // OUT: address of first instruction to execute
       Addr brkbase;      // OUT: base address of brk segment
@@ -99,9 +76,10 @@ typedef
 
 // Do a number of appropriate checks to see if the file looks executable by
 // the kernel: ie. it's a file, it's readable and executable, and it's in
-// either ELF or "#!" format.  On success, 'out_fd' gets the fd of the file
+// either binary or "#!" format.  On success, 'out_fd' gets the fd of the file
 // if it's non-NULL.  Otherwise the fd is closed.
-extern SysRes VG_(pre_exec_check)(const HChar* exe_name, Int* out_fd);
+extern SysRes VG_(pre_exec_check)(const HChar* exe_name, Int* out_fd,
+                                  Bool allow_setuid);
 
 // Does everything short of actually running 'exe': finds the file,
 // checks execute permissions, sets up interpreter if program is a script, 
@@ -109,21 +87,6 @@ extern SysRes VG_(pre_exec_check)(const HChar* exe_name, Int* out_fd);
 // the program.
 extern Int VG_(do_exec)(const HChar* exe, ExeInfo* info);
 
-/*------------------------------------------------------------*/
-/*--- Finding and dealing with auxv                        ---*/
-/*------------------------------------------------------------*/
-
-struct ume_auxv
-{
-   Word a_type;
-   union {
-      void *a_ptr;
-      Word a_val;
-   } u;
-};
-
-extern struct ume_auxv *VG_(find_auxv)(UWord* orig_esp);
-
 #endif /* __PUB_CORE_UME_H */
 
 /*--------------------------------------------------------------------*/
