@@ -7,7 +7,7 @@
    This file is part of Valgrind, a dynamic binary instrumentation
    framework.
 
-   Copyright (C) 2000-2009 Julian Seward 
+   Copyright (C) 2000-2010 Julian Seward 
       jseward@acm.org
 
    This program is free software; you can redistribute it and/or
@@ -62,6 +62,12 @@ typedef unsigned int vki_u32;
 #define VKI_MAX_PAGE_SHIFT	VKI_PAGE_SHIFT
 #define VKI_MAX_PAGE_SIZE	VKI_PAGE_SIZE
 
+//----------------------------------------------------------------------
+// From linux-2.6.35.4/arch/x86/include/asm/shmparam.h
+//----------------------------------------------------------------------
+
+#define VKI_SHMLBA  VKI_PAGE_SIZE
+
 //----------------------------------------------------------------------
 // From linux-2.6.9/include/asm-x86_64/signal.h
 //----------------------------------------------------------------------
