@@ -7,7 +7,7 @@
    This file is part of Valgrind, a dynamic binary instrumentation
    framework.
 
-   Copyright (C) 2000-2007 Julian Seward 
+   Copyright (C) 2000-2010 Julian Seward 
       jseward@acm.org
 
    This program is free software; you can redistribute it and/or
@@ -87,10 +87,40 @@
 #  include "vki-posixtypes-ppc32-linux.h"
 #elif defined(VGA_ppc64)
 #  include "vki-posixtypes-ppc64-linux.h"
+#elif defined(VGA_arm)
+#  include "vki-posixtypes-arm-linux.h"
 #else
 #  error Unknown platform
 #endif
 
+//----------------------------------------------------------------------
+// VKI_STATIC_ASSERT(). Inspired by BUILD_BUG_ON() from
+// linux-2.6.34/include/linux/kernel.h
+//----------------------------------------------------------------------
+
+/*
+ * Evaluates to zero if 'expr' is true and forces a compilation error if
+ * 'expr' is false. Can be used in a context where no comma expressions
+ * are allowed.
+ */
+#ifdef __cplusplus
+template <bool b> struct vki_static_assert { int m_bitfield:(2*b-1); };
+#define VKI_STATIC_ASSERT(expr)                         \
+    (sizeof(vki_static_assert<(expr)>) - sizeof(int))
+#else
+#define VKI_STATIC_ASSERT(expr) (sizeof(struct { int:-!(expr); }))
+#endif
+
+//----------------------------------------------------------------------
+// Based on _IOC_TYPECHECK() from linux-2.6.34/asm-generic/ioctl.h
+//----------------------------------------------------------------------
+
+/* provoke compile error for invalid uses of size argument */
+#define _VKI_IOC_TYPECHECK(t)                                           \
+    (VKI_STATIC_ASSERT((sizeof(t) == sizeof(t[1])                       \
+                        && sizeof(t) < (1 << _VKI_IOC_SIZEBITS)))       \
+     + sizeof(t))
+
 //----------------------------------------------------------------------
 // From linux-2.6.8.1/include/linux/compiler.h
 //----------------------------------------------------------------------
@@ -169,6 +199,8 @@ typedef unsigned int	        vki_uint;
 #  include "vki-ppc32-linux.h"
 #elif defined(VGA_ppc64)
 #  include "vki-ppc64-linux.h"
+#elif defined(VGA_arm)
+#  include "vki-arm-linux.h"
 #else
 #  error Unknown platform
 #endif
@@ -179,6 +211,8 @@ typedef unsigned int	        vki_uint;
 
 typedef		__vki_s32	vki_int32_t;
 
+typedef		__vki_u8	vki_uint8_t;
+typedef		__vki_u16	vki_uint16_t;
 typedef		__vki_u32	vki_uint32_t;
 
 //----------------------------------------------------------------------
@@ -212,6 +246,11 @@ struct vki_sysinfo {
 // From linux-2.6.8.1/include/linux/time.h
 //----------------------------------------------------------------------
 
+#define VKI_CLOCK_REALTIME            0
+#define VKI_CLOCK_MONOTONIC           1
+#define VKI_CLOCK_PROCESS_CPUTIME_ID  2
+#define VKI_CLOCK_THREAD_CPUTIME_ID   3
+
 struct vki_timespec {
 	vki_time_t	tv_sec;		/* seconds */
 	long		tv_nsec;	/* nanoseconds */
@@ -270,14 +309,17 @@ struct vki_timex {
 	int  :32; int  :32; int  :32; int  :32;
 };
 
-//#define ADJ_OFFSET		0x0001	/* time offset */
-#define ADJ_FREQUENCY		0x0002	/* frequency offset */
-#define ADJ_MAXERROR		0x0004	/* maximum time error */
-#define ADJ_ESTERROR		0x0008	/* estimated time error */
-#define ADJ_STATUS		0x0010	/* clock status */
-#define ADJ_TIMECONST		0x0020	/* pll time constant */
-#define ADJ_TICK		0x4000	/* tick value */
-//#define ADJ_OFFSET_SINGLESHOT	0x8001	/* old-fashioned adjtime */
+#define VKI_ADJ_OFFSET			0x0001	/* time offset */
+#define VKI_ADJ_FREQUENCY		0x0002	/* frequency offset */
+#define VKI_ADJ_MAXERROR		0x0004	/* maximum time error */
+#define VKI_ADJ_ESTERROR		0x0008	/* estimated time error */
+#define VKI_ADJ_STATUS			0x0010	/* clock status */
+#define VKI_ADJ_TIMECONST		0x0020	/* pll time constant */
+#define VKI_ADJ_TAI			0x0080	/* set TAI offset */
+#define VKI_ADJ_TICK			0x4000	/* tick value */
+#define VKI_ADJ_ADJTIME			0x8000	/* switch between adjtime/adjtimex modes */
+//#define VKI_ADJ_OFFSET_SINGLESHOT	0x8001	/* old-fashioned adjtime */
+#define VKI_ADJ_OFFSET_READONLY		0x2000	/* read-only adjtime */
 
 //----------------------------------------------------------------------
 // From linux-2.6.8.1/include/linux/times.h
@@ -518,6 +560,7 @@ typedef struct vki_sigevent {
 #define VKI_SYS_GETSOCKOPT	15	/* sys_getsockopt(2)		*/
 #define VKI_SYS_SENDMSG		16	/* sys_sendmsg(2)		*/
 #define VKI_SYS_RECVMSG		17	/* sys_recvmsg(2)		*/
+#define VKI_SYS_ACCEPT4		18	/* sys_accept4(2)		*/
 
 enum vki_sock_type {
 	VKI_SOCK_STREAM	= 1,
@@ -1143,7 +1186,7 @@ struct  vki_seminfo {
 #define VKI_MREMAP_FIXED	2
 
 //----------------------------------------------------------------------
-// From linux-2.6.10-rc3-mm1/include/linux/futex.h
+// From linux-2.6.31-rc4/include/linux/futex.h
 //----------------------------------------------------------------------
 
 #define VKI_FUTEX_WAIT (0)
@@ -1151,7 +1194,16 @@ struct  vki_seminfo {
 #define VKI_FUTEX_FD (2)
 #define VKI_FUTEX_REQUEUE (3)
 #define VKI_FUTEX_CMP_REQUEUE (4)
+#define VKI_FUTEX_WAKE_OP (5)
+#define VKI_FUTEX_LOCK_PI (6)
+#define VKI_FUTEX_UNLOCK_PI (7)
+#define VKI_FUTEX_TRYLOCK_PI (8)
+#define VKI_FUTEX_WAIT_BITSET (9)
+#define VKI_FUTEX_WAKE_BITSET (10)
+#define VKI_FUTEX_WAIT_REQUEUE_PI (11)
+#define VKI_FUTEX_CMP_REQUEUE_PI (12)
 #define VKI_FUTEX_PRIVATE_FLAG (128)
+#define VKI_FUTEX_CLOCK_REALTIME (256)
 
 struct vki_robust_list {
 	struct vki_robust_list __user *next;
@@ -1244,10 +1296,17 @@ struct vki_dirent {
 // From linux-2.6.8.1/include/linux/fcntl.h
 //----------------------------------------------------------------------
 
-#define VKI_F_SETLEASE	(VKI_F_LINUX_SPECIFIC_BASE+0)
-#define VKI_F_GETLEASE	(VKI_F_LINUX_SPECIFIC_BASE+1)
+#define VKI_F_SETLEASE      (VKI_F_LINUX_SPECIFIC_BASE + 0)
+#define VKI_F_GETLEASE      (VKI_F_LINUX_SPECIFIC_BASE + 1)
 
-#define VKI_F_NOTIFY	(VKI_F_LINUX_SPECIFIC_BASE+2)
+#define VKI_F_CANCELLK      (VKI_F_LINUX_SPECIFIC_BASE + 5)
+
+#define VKI_F_DUPFD_CLOEXEC (VKI_F_LINUX_SPECIFIC_BASE + 6)
+
+#define VKI_F_NOTIFY        (VKI_F_LINUX_SPECIFIC_BASE + 2)
+
+#define VKI_F_SETPIPE_SZ    (VKI_F_LINUX_SPECIFIC_BASE + 7)
+#define VKI_F_GETPIPE_SZ    (VKI_F_LINUX_SPECIFIC_BASE + 8)
 
 //----------------------------------------------------------------------
 // From linux-2.6.8.1/include/linux/sysctl.h
@@ -1272,6 +1331,10 @@ typedef unsigned long	vki_aio_context_t;
 enum {
 	VKI_IOCB_CMD_PREAD = 0,
 	VKI_IOCB_CMD_PWRITE = 1,
+	VKI_IOCB_CMD_FSYNC = 2,
+	VKI_IOCB_CMD_FDSYNC = 3,
+	VKI_IOCB_CMD_PREADV = 7,
+	VKI_IOCB_CMD_PWRITEV = 8,
 };
 
 /* read() from /dev/aio returns these structures. */
@@ -1388,6 +1451,7 @@ struct vki_shmid_ds {
 };
 
 #define VKI_SHM_RDONLY  010000  /* read-only access */
+#define VKI_SHM_RND     020000  /* round attach address to SHMLBA boundary */
 
 #define VKI_SHM_STAT 	13
 #define VKI_SHM_INFO 	14
@@ -1840,6 +1904,7 @@ struct vki_cdrom_generic_command
 #define VKI_SNDCTL_DSP_SETFRAGMENT	_VKI_SIOWR('P',10, int)
 
 #define VKI_SNDCTL_DSP_GETFMTS		_VKI_SIOR ('P',11, int) /* Returns a mask */
+#define VKI_SNDCTL_DSP_SETFMT		_VKI_SIOWR('P', 5, int) /* Selects ONE fmt */
 
 typedef struct vki_audio_buf_info {
 			int fragments;	/* # of available fragments (partially usend ones not counted) */
@@ -1852,7 +1917,7 @@ typedef struct vki_audio_buf_info {
 
 #define VKI_SNDCTL_DSP_GETOSPACE	_VKI_SIOR ('P',12, vki_audio_buf_info)
 #define VKI_SNDCTL_DSP_GETISPACE	_VKI_SIOR ('P',13, vki_audio_buf_info)
-//#define VKI_SNDCTL_DSP_NONBLOCK	_VKI_SIO  ('P',14)
+#define VKI_SNDCTL_DSP_NONBLOCK		_VKI_SIO  ('P',14)
 #define VKI_SNDCTL_DSP_GETCAPS		_VKI_SIOR ('P',15, int)
 
 #define VKI_SNDCTL_DSP_GETTRIGGER	_VKI_SIOR ('P',16, int)
@@ -2127,8 +2192,10 @@ enum {
 	VKI_SNDRV_PCM_IOCTL_START = _VKI_IO('A', 0x42),
 	VKI_SNDRV_PCM_IOCTL_DROP = _VKI_IO('A', 0x43),
 	VKI_SNDRV_PCM_IOCTL_DRAIN = _VKI_IO('A', 0x44),
+	VKI_SNDRV_PCM_IOCTL_PAUSE = _VKI_IOW('A', 0x45, int),
 	VKI_SNDRV_PCM_IOCTL_RESUME = _VKI_IO('A', 0x47),
 	VKI_SNDRV_PCM_IOCTL_XRUN = _VKI_IO('A', 0x48),
+	VKI_SNDRV_PCM_IOCTL_LINK = _VKI_IOW('A', 0x60, int),
 	VKI_SNDRV_PCM_IOCTL_UNLINK = _VKI_IO('A', 0x61),
 };
 
@@ -2318,11 +2385,26 @@ struct vki_usbdevfs_ioctl {
 #define VKI_USBDEVFS_BULK              _VKI_IOWR('U', 2, struct vki_usbdevfs_bulktransfer)
 #define VKI_USBDEVFS_GETDRIVER         _VKI_IOW('U', 8, struct vki_usbdevfs_getdriver)
 #define VKI_USBDEVFS_SUBMITURB         _VKI_IOR('U', 10, struct vki_usbdevfs_urb)
+#define VKI_USBDEVFS_DISCARDURB        _VKI_IO('U', 11)
 #define VKI_USBDEVFS_REAPURB           _VKI_IOW('U', 12, void *)
 #define VKI_USBDEVFS_REAPURBNDELAY     _VKI_IOW('U', 13, void *)
 #define VKI_USBDEVFS_CONNECTINFO       _VKI_IOW('U', 17, struct vki_usbdevfs_connectinfo)
 #define VKI_USBDEVFS_IOCTL             _VKI_IOWR('U', 18, struct vki_usbdevfs_ioctl)
 
+#define VKI_USBDEVFS_URB_TYPE_ISO              0
+#define VKI_USBDEVFS_URB_TYPE_INTERRUPT        1
+#define VKI_USBDEVFS_URB_TYPE_CONTROL          2
+#define VKI_USBDEVFS_URB_TYPE_BULK             3
+
+// [[this is missing in usbdevice_fs.h]]
+struct vki_usbdevfs_setuppacket {
+       __vki_u8 bRequestType;
+       __vki_u8 bRequest;
+       __vki_u16 wValue;
+       __vki_u16 wIndex;
+       __vki_u16 wLength;
+};
+
 //----------------------------------------------------------------------
 // From linux-2.6.20.1/include/linux/i2c.h
 //----------------------------------------------------------------------
@@ -2370,6 +2452,271 @@ typedef vki_int32_t vki_key_serial_t;
 /* key handle permissions mask */
 typedef vki_uint32_t vki_key_perm_t;
 
+//----------------------------------------------------------------------
+// From linux-2.6.24.7/include/linux/wireless.h
+// (wireless extensions version 22, 2007-03-16)
+//----------------------------------------------------------------------
+
+/*
+ * [[Wireless extensions ioctls.]]
+ */
+
+/* Wireless Identification */
+#define VKI_SIOCSIWCOMMIT	0x8B00	/* Commit pending changes to driver */
+#define VKI_SIOCGIWNAME		0x8B01	/* get name == wireless protocol */
+
+/* Basic operations */
+#define VKI_SIOCSIWNWID		0x8B02	/* set network id (pre-802.11) */
+#define VKI_SIOCGIWNWID		0x8B03	/* get network id (the cell) */
+#define VKI_SIOCSIWFREQ		0x8B04	/* set channel/frequency (Hz) */
+#define VKI_SIOCGIWFREQ		0x8B05	/* get channel/frequency (Hz) */
+#define VKI_SIOCSIWMODE		0x8B06	/* set operation mode */
+#define VKI_SIOCGIWMODE		0x8B07	/* get operation mode */
+#define VKI_SIOCSIWSENS		0x8B08	/* set sensitivity (dBm) */
+#define VKI_SIOCGIWSENS		0x8B09	/* get sensitivity (dBm) */
+
+/* Informative stuff */
+#define VKI_SIOCSIWRANGE	0x8B0A	/* Unused */
+#define VKI_SIOCGIWRANGE	0x8B0B	/* Get range of parameters */
+#define VKI_SIOCSIWPRIV		0x8B0C	/* Unused */
+#define VKI_SIOCGIWPRIV		0x8B0D	/* get private ioctl interface info */
+#define VKI_SIOCSIWSTATS	0x8B0E	/* Unused */
+#define VKI_SIOCGIWSTATS	0x8B0F	/* Get /proc/net/wireless stats */
+
+/* Spy support (statistics per MAC address - used for Mobile IP support) */
+#define VKI_SIOCSIWSPY		0x8B10	/* set spy addresses */
+#define VKI_SIOCGIWSPY		0x8B11	/* get spy info (quality of link) */
+#define VKI_SIOCSIWTHRSPY	0x8B12	/* set spy threshold (spy event) */
+#define VKI_SIOCGIWTHRSPY	0x8B13	/* get spy threshold */
+
+/* Access Point manipulation */
+#define VKI_SIOCSIWAP		0x8B14	/* set access point MAC addresses */
+#define VKI_SIOCGIWAP		0x8B15	/* get access point MAC addresses */
+#define VKI_SIOCGIWAPLIST	0x8B17	/* Deprecated in favor of scanning */
+#define VKI_SIOCSIWSCAN         0x8B18	/* trigger scanning (list cells) */
+#define VKI_SIOCGIWSCAN         0x8B19	/* get scanning results */
+
+/* 802.11 specific support */
+#define VKI_SIOCSIWESSID	0x8B1A	/* set ESSID (network name) */
+#define VKI_SIOCGIWESSID	0x8B1B	/* get ESSID */
+#define VKI_SIOCSIWNICKN	0x8B1C	/* set node name/nickname */
+#define VKI_SIOCGIWNICKN	0x8B1D	/* get node name/nickname */
+
+/* Other parameters useful in 802.11 and some other devices */
+#define VKI_SIOCSIWRATE		0x8B20	/* set default bit rate (bps) */
+#define VKI_SIOCGIWRATE		0x8B21	/* get default bit rate (bps) */
+#define VKI_SIOCSIWRTS		0x8B22	/* set RTS/CTS threshold (bytes) */
+#define VKI_SIOCGIWRTS		0x8B23	/* get RTS/CTS threshold (bytes) */
+#define VKI_SIOCSIWFRAG		0x8B24	/* set fragmentation thr (bytes) */
+#define VKI_SIOCGIWFRAG		0x8B25	/* get fragmentation thr (bytes) */
+#define VKI_SIOCSIWTXPOW	0x8B26	/* set transmit power (dBm) */
+#define VKI_SIOCGIWTXPOW	0x8B27	/* get transmit power (dBm) */
+#define VKI_SIOCSIWRETRY	0x8B28	/* set retry limits and lifetime */
+#define VKI_SIOCGIWRETRY	0x8B29	/* get retry limits and lifetime */
+
+/* Encoding stuff (scrambling, hardware security, WEP...) */
+#define VKI_SIOCSIWENCODE	0x8B2A	/* set encoding token & mode */
+#define VKI_SIOCGIWENCODE	0x8B2B	/* get encoding token & mode */
+
+/* Power saving stuff (power management, unicast and multicast) */
+#define VKI_SIOCSIWPOWER	0x8B2C	/* set Power Management settings */
+#define VKI_SIOCGIWPOWER	0x8B2D	/* get Power Management settings */
+
+/* WPA : Generic IEEE 802.11 informatiom element (e.g., for WPA/RSN/WMM). */
+#define VKI_SIOCSIWGENIE	0x8B30		/* set generic IE */
+#define VKI_SIOCGIWGENIE	0x8B31		/* get generic IE */
+
+/* WPA : IEEE 802.11 MLME requests */
+#define VKI_SIOCSIWMLME		0x8B16	/* request MLME operation; uses
+					 * struct iw_mlme */
+/* WPA : Authentication mode parameters */
+#define VKI_SIOCSIWAUTH		0x8B32	/* set authentication mode params */
+#define VKI_SIOCGIWAUTH		0x8B33	/* get authentication mode params */
+
+/* WPA : Extended version of encoding configuration */
+#define VKI_SIOCSIWENCODEEXT	0x8B34	/* set encoding token & mode */
+#define VKI_SIOCGIWENCODEEXT	0x8B35	/* get encoding token & mode */
+
+/* WPA2 : PMKSA cache management */
+#define VKI_SIOCSIWPMKSA	0x8B36	/* PMKSA cache operation */
+
+/*
+ * [[Payload for the wireless extensions ioctls.]]
+ */
+
+struct	vki_iw_param
+{
+  __vki_s32	value;		/* The value of the parameter itself */
+  __vki_u8	fixed;		/* Hardware should not use auto select */
+  __vki_u8	disabled;	/* Disable the feature */
+  __vki_u16	flags;		/* Various specifc flags (if any) */
+};
+
+struct	vki_iw_point
+{
+  void __user	*pointer;	/* Pointer to the data  (in user space) */
+  __vki_u16	length;		/* number of fields or size in bytes */
+  __vki_u16	flags;		/* Optional params */
+};
+
+struct	vki_iw_freq
+{
+	__vki_s32	m;		/* Mantissa */
+	__vki_s16	e;		/* Exponent */
+	__vki_u8	i;		/* List index (when in range struct) */
+	__vki_u8	flags;		/* Flags (fixed/auto) */
+};
+
+struct	vki_iw_quality
+{
+	__vki_u8	qual;		/* link quality (%retries, SNR,
+					   %missed beacons or better...) */
+	__vki_u8	level;		/* signal level (dBm) */
+	__vki_u8	noise;		/* noise level (dBm) */
+	__vki_u8	updated;	/* Flags to know if updated */
+};
+
+union	vki_iwreq_data
+{
+	/* Config - generic */
+	char		name[VKI_IFNAMSIZ];
+	/* Name : used to verify the presence of  wireless extensions.
+	 * Name of the protocol/provider... */
+
+	struct vki_iw_point	essid;	/* Extended network name */
+	struct vki_iw_param	nwid;	/* network id (or domain - the cell) */
+	struct vki_iw_freq	freq;	/* frequency or channel :
+					 * 0-1000 = channel
+					 * > 1000 = frequency in Hz */
+
+	struct vki_iw_param	sens;	/* signal level threshold */
+	struct vki_iw_param	bitrate;/* default bit rate */
+	struct vki_iw_param	txpower;/* default transmit power */
+	struct vki_iw_param	rts;	/* RTS threshold threshold */
+	struct vki_iw_param	frag;	/* Fragmentation threshold */
+	__vki_u32		mode;	/* Operation mode */
+	struct vki_iw_param	retry;	/* Retry limits & lifetime */
+
+	struct vki_iw_point	encoding; /* Encoding stuff : tokens */
+	struct vki_iw_param	power;	/* PM duration/timeout */
+	struct vki_iw_quality	qual;	/* Quality part of statistics */
+
+	struct vki_sockaddr ap_addr;	/* Access point address */
+	struct vki_sockaddr addr;	/* Destination address (hw/mac) */
+
+	struct vki_iw_param	param;	/* Other small parameters */
+	struct vki_iw_point	data;	/* Other large parameters */
+};
+
+struct	vki_iwreq 
+{
+	union
+	{
+		char ifrn_name[VKI_IFNAMSIZ];	/* if name, e.g. "eth0" */
+	} ifr_ifrn;
+
+	/* Data part (defined just above) */
+	union	vki_iwreq_data	u;
+};
+
+/*--------------------------------------------------------------------*/
+// From linux-2.6.31.5/include/linux/perf_counter.h
+/*--------------------------------------------------------------------*/
+
+struct vki_perf_counter_attr {
+
+	/*
+	 * Major type: hardware/software/tracepoint/etc.
+	 */
+	__vki_u32			type;
+
+	/*
+	 * Size of the attr structure, for fwd/bwd compat.
+	 */
+	__vki_u32			size;
+
+	/*
+	 * Type specific configuration information.
+	 */
+	__vki_u64			config;
+
+	union {
+		__vki_u64		sample_period;
+		__vki_u64		sample_freq;
+	};
+
+	__vki_u64			sample_type;
+	__vki_u64			read_format;
+
+	__vki_u64			disabled       :  1, /* off by default        */
+					inherit	       :  1, /* children inherit it   */
+					pinned	       :  1, /* must always be on PMU */
+					exclusive      :  1, /* only group on PMU     */
+					exclude_user   :  1, /* don't count user      */
+					exclude_kernel :  1, /* ditto kernel          */
+					exclude_hv     :  1, /* ditto hypervisor      */
+					exclude_idle   :  1, /* don't count when idle */
+					mmap           :  1, /* include mmap data     */
+					comm	       :  1, /* include comm data     */
+					freq           :  1, /* use freq, not period  */
+					inherit_stat   :  1, /* per task counts       */
+					enable_on_exec :  1, /* next exec enables     */
+					task           :  1, /* trace fork/exit       */
+
+					__reserved_1   : 50;
+
+	__vki_u32			wakeup_events;	/* wakeup every n events */
+	__vki_u32			__reserved_2;
+
+	__vki_u64			__reserved_3;
+};
+
+/*--------------------------------------------------------------------*/
+// From linux-2.6.32.4/include/linux/getcpu.h
+/*--------------------------------------------------------------------*/
+
+struct vki_getcpu_cache {
+	unsigned long blob[128 / sizeof(long)];
+};
+
+//----------------------------------------------------------------------
+// From linux-2.6.33.3/include/linux/input.h
+//----------------------------------------------------------------------
+
+/*
+ * IOCTLs (0x00 - 0x7f)
+ */
+
+#define VKI_EVIOCGNAME(len)	_VKI_IOC(_VKI_IOC_READ, 'E', 0x06, len)		/* get device name */
+#define VKI_EVIOCGPHYS(len)	_VKI_IOC(_VKI_IOC_READ, 'E', 0x07, len)		/* get physical location */
+#define VKI_EVIOCGUNIQ(len)	_VKI_IOC(_VKI_IOC_READ, 'E', 0x08, len)		/* get unique identifier */
+
+#define VKI_EVIOCGKEY(len)	_VKI_IOC(_VKI_IOC_READ, 'E', 0x18, len)		/* get global keystate */
+#define VKI_EVIOCGLED(len)	_VKI_IOC(_VKI_IOC_READ, 'E', 0x19, len)		/* get all LEDs */
+#define VKI_EVIOCGSND(len)	_VKI_IOC(_VKI_IOC_READ, 'E', 0x1a, len)		/* get all sounds status */
+#define VKI_EVIOCGSW(len)	_VKI_IOC(_VKI_IOC_READ, 'E', 0x1b, len)		/* get all switch states */
+
+#define VKI_EVIOCGBIT(ev,len)	_VKI_IOC(_VKI_IOC_READ, 'E', 0x20 + ev, len)	/* get event bits */
+
+/*
+ * Event types
+ */
+
+#define VKI_EV_SYN		0x00
+#define VKI_EV_KEY		0x01
+#define VKI_EV_REL		0x02
+#define VKI_EV_ABS		0x03
+#define VKI_EV_MSC		0x04
+#define VKI_EV_SW		0x05
+#define VKI_EV_LED		0x11
+#define VKI_EV_SND		0x12
+#define VKI_EV_REP		0x14
+#define VKI_EV_FF		0x15
+#define VKI_EV_PWR		0x16
+#define VKI_EV_FF_STATUS	0x17
+#define VKI_EV_MAX		0x1f
+#define VKI_EV_CNT		(VKI_EV_MAX+1)
+
 #endif // __VKI_LINUX_H
 
 /*--------------------------------------------------------------------*/
